@@ -23,8 +23,9 @@
 #'  for the gaussian surface or volume smoothing kernel, in mm. Default: \code{5}
 #'  for cortex (surface) and \code{3} for subcortex (volume).
 #' @param surfL_fname,surfR_fname (Required if the corresponding cortex is 
-#'  present) Surface GIFTI files for the left and right cortical surfaces. If not
-#'  provided, the default surfaces will be used.
+#'  present) Surface GIFTI files for the left and right cortical surfaces. If
+#'  not provided, the surfaces in \code{x} are used, but if those are also not
+#'  present, the default surfaces will be used.
 #' @param cerebellum_fname (Optional) Surface GIFTI file for the cerebellar surface
 #' @param subcortical_zeroes_as_NA,cortical_zeroes_as_NA Should zero-values in 
 #'  the subcortical volume or cortex be treated as NA? Default: \code{FALSE}.
@@ -62,6 +63,13 @@ smooth_cifti <- function(
       x_extn <- "dscalar.nii"
     }
 
+    res <- infer_resolution(x)
+    if (any(res %in% c(NA, NaN))) {
+      stop("Cannot smooth because the resolution could not be inferred.")
+    } else if (res[1] != res[2]) {
+      stop("Cannot smooth because the left and right cortex have unequal inferred resolutions: ", res[1], " and ", res[2], ".")
+    }
+
     # Write out the CIFTI.
     cifti_original_fname <- file.path(tempdir(), paste0("to_smooth.", x_extn))
     write_cifti(x, cifti_original_fname, verbose=FALSE)
