@@ -2,7 +2,7 @@
 #'
 #' Summary method for class \code{"xifti"}
 #'
-#' @param object Object of class \code{"xifti"}. 
+#' @param object Object of class \code{"xifti"}.
 #' @param ... further arguments passed to or from other methods.
 #' @export
 #' @method summary xifti
@@ -21,18 +21,18 @@ summary.xifti <- function(object, ...) {
 
   # Re-name the brainstructures
   bs_nice <- c(
-    cortex_left = "left cortex", 
-    cortex_right = "right cortex", 
+    cortex_left = "left cortex",
+    cortex_right = "right cortex",
     subcort = "subcortex"
   )
   names(x$brainstructure) <- bs_nice[names(x$brainstructure)]
   names(x$verts_per_bs) <- bs_nice[names(x$verts_per_bs)]
   bs_nice <- c(
-    cortex_left = "left", 
+    cortex_left = "left",
     cortex_right = "right"
   )
   names(x$surf) <- bs_nice[names(x$surf)]
-  
+
   # Handle NULL/0/etc.
   x$verts_per_bs[vapply(x$vertx_per_bs, is.null, FALSE)] <- 0
   x["verts_per_bs"] <- list(unlist(x$verts_per_bs))
@@ -43,6 +43,12 @@ summary.xifti <- function(object, ...) {
   if (dim(x$subcort_labels) < 1) {
     x["subcort_labels"] <- list(NULL)
   }
+  if (!is.null(x$medial_wall_mask$left)) {
+    x$medial_wall_mask$left[is.na(x$medial_wall_mask$left)] <- 0
+  }
+  if (!is.null(x$medial_wall_mask$right)) {
+    x$medial_wall_mask$right[is.na(x$medial_wall_mask$right)] <- 0
+  }
 
   # Add intent-specific entries
   if (!is.null(x$intent)) {
@@ -64,7 +70,7 @@ summary.xifti <- function(object, ...) {
 
 #' @rdname summary.xifti
 #' @export
-#' 
+#'
 #' @inheritParams x_Param_xifti
 #' @method print summary.xifti
 print.summary.xifti <- function(x, ...) {
@@ -73,14 +79,14 @@ print.summary.xifti <- function(x, ...) {
 
   if (!is.null(x$intent)) {
     cat(paste0(
-      "Intent:           ", x$intent, " (", 
-      c("dtseries", "dscalar", "dlabel")[match(x$intent, c(3002, 3006, 3007))], 
+      "Intent:           ", x$intent, " (",
+      c("dtseries", "dscalar", "dlabel")[match(x$intent, c(3002, 3006, 3007))],
       ")\n"
     ))
 
     if (x$intent == 3002) {
       cat(paste0(
-        "- time step       ", x$time["step"], " (", x$time["unit"], 
+        "- time step       ", x$time["step"], " (", x$time["unit"],
         ifelse(x$time["unit"]=="hertz", "", "s"), ")\n"
       ))
       cat(paste0(
@@ -89,14 +95,14 @@ print.summary.xifti <- function(x, ...) {
     } else {
       few_names <- x$names[seq(min(5, length(x$names)))]
       cat(paste0(
-        "- names           \"", paste0(few_names, collapse="\", \""), 
+        "- names           \"", paste0(few_names, collapse="\", \""),
         ifelse(length(few_names) < length(x$names), "\", ... ", "\""), "\n"
       ))
     }
   }
 
   cat(paste0(
-    "Measurements:     ", x$measurements, 
+    "Measurements:     ", x$measurements,
     " column", ifelse(x$measurements>1, "s", ""), "\n"
   ))
 
@@ -148,11 +154,11 @@ print.summary.xifti <- function(x, ...) {
     labs2 <- unique(gsub("-.*", "", names(x$subcort_labels)))
     for (ii in seq(length(labs2))) {
       labs2_ii <- labs2[ii]
-      if (labs2_ii != "Brain Stem") { 
-        labs2_ii <- paste0(labs2_ii, c("-L", "-R")) 
+      if (labs2_ii != "Brain Stem") {
+        labs2_ii <- paste0(labs2_ii, c("-L", "-R"))
       }
       cat(paste0(
-        blank, "  ", 
+        blank, "  ",
         paste(labs2_ii, x$subcort_labels[labs2_ii], sep= " (", collapse="), "),
         ")", ifelse(ii == length(labs2), ".", ",")
       ))
@@ -167,7 +173,7 @@ print.summary.xifti <- function(x, ...) {
     #   labs_ii <- pmin(labs_ii, n_labs)
     #   labs_ii <- x$subcort_labels[labs_ii]
     #   cat(paste0(
-    #     blank, paste(names(labs_ii), labs_ii, sep= " (", collapse="), "), 
+    #     blank, paste(names(labs_ii), labs_ii, sep= " (", collapse="), "),
     #     ")", ifelse(ii == q, ".", ",")
     #   ))
     #   cat("\n")
@@ -179,7 +185,7 @@ print.summary.xifti <- function(x, ...) {
 
 #' @rdname summary.xifti
 #' @export
-#' 
+#'
 #' @method print xifti
 print.xifti <- function(x, ...) {
   print.summary.xifti(summary(x))
@@ -192,7 +198,7 @@ print.xifti <- function(x, ...) {
 #' @return Character file extension of CIFTI file, e.g. "dtseries.nii", "dscalar.nii"
 #'
 #' @keywords internal
-#' 
+#'
 get_cifti_extn <- function(cifti_fname) {
   fname_parts <- unlist(strsplit(basename(cifti_fname), split=".", fixed = TRUE)) #split by "."
   extn <- paste(rev(fname_parts)[c(2,1)], collapse=".") #'dtseries.nii", "dscalar.nii", etc.
@@ -200,13 +206,13 @@ get_cifti_extn <- function(cifti_fname) {
 }
 
 #' Get CIFTI component suffix default
-#' 
+#'
 #' Get the default file name suffix for a certain type of GIFTI/NIFTI file
 #'
-#' @param label the file type: one of 
-#'  \code{"cortexL"}, \code{"cortexR"}, 
+#' @param label the file type: one of
+#'  \code{"cortexL"}, \code{"cortexR"},
 #'  \code{"subcortVol"}, \code{"subcortLabs"},
-#'  \code{"ROIcortexL"}, \code{"ROIcortexR"}, 
+#'  \code{"ROIcortexL"}, \code{"ROIcortexR"},
 #'  \code{"ROIsubcortVol"},
 #'  \code{"validROIcortexL"}, or \code{"validROIcortexR"}.
 #' @param GIFTI_type Used to make the suffix. Default: \code{"func"}.
@@ -214,7 +220,7 @@ get_cifti_extn <- function(cifti_fname) {
 #' @return The default file name suffix
 #'
 #' @keywords internal
-#' 
+#'
 cifti_component_suffix <- function(label, GIFTI_type="func") {
   label <- match.arg(label, c(
     "cortexL", "cortexR", "subcortVol", "subcortLabs",
@@ -232,7 +238,7 @@ cifti_component_suffix <- function(label, GIFTI_type="func") {
 }
 
 #' Get resampled file name default
-#' 
+#'
 #' Get the default file name for a resampled file.
 #'
 #' @param original_fname The original file name
@@ -241,7 +247,7 @@ cifti_component_suffix <- function(label, GIFTI_type="func") {
 #' @return The default file name
 #'
 #' @keywords internal
-#' 
+#'
 resample_cifti_default_fname <- function(original_fname, resamp_res) {
   stopifnot(!is.null(original_fname))
   bname <- basename(original_fname)
@@ -249,16 +255,16 @@ resample_cifti_default_fname <- function(original_fname, resamp_res) {
 }
 
 #' Unmask cortex
-#' 
+#'
 #' Get cortex data with medial wall vertices
-#' 
+#'
 #' @param cortex V vertices x T measurements matrix
 #' @param mwall Logical vector with T \code{TRUE} values.
 #' @param mwall_fill The fill value to use for medial wall vertices.
 #' @return The unmasked cortex data
-#' 
+#'
 #' @export
-#' 
+#'
 unmask_cortex <- function(cortex, mwall, mwall_fill=NA) {
   cdat <- matrix(mwall_fill, nrow=length(mwall),  ncol=ncol(cortex))
   cdat[mwall,] <- cortex
@@ -266,20 +272,20 @@ unmask_cortex <- function(cortex, mwall, mwall_fill=NA) {
 }
 
 #' Counts the number of rows (vertices + voxels) in a \code{"xifti"}.
-#' 
+#'
 #' Counts the number of data locations in the \code{"xifti"} data. Doesn't bother
 #'  to validate the input.
-#' 
+#'
 #' @inheritParams xifti_Param
 #' @return The number of rows in the \code{"xifti"} data.
-#' 
+#'
 #' @keywords internal
-#' 
+#'
 nrow_xifti <- function(xifti) {
   # This function is generic because nrow(xii) will actually use dim(xii)[1]
   #   which calls this function.
   bs_present <- !vapply(xifti$data, is.null, FALSE)
-  if (!any(bs_present)) { 
+  if (!any(bs_present)) {
     return(0)
   } else {
     return(sum(vapply(xifti$data[bs_present], nrow, 0)))
@@ -287,18 +293,18 @@ nrow_xifti <- function(xifti) {
 }
 
 #' Counts the number of columns in a \code{"xifti"}.
-#' 
+#'
 #' Counts the number of columns in the \code{"xifti"} data. Doesn't bother
 #'  to validate the input.
-#' 
+#'
 #' @inheritParams x_Param_xifti
 #' @return The number of columns in the \code{"xifti"} data.
-#' 
+#'
 #' @keywords internal
-#' 
+#'
 ncol_xifti <- function(xifti) {
   bs_present <- !vapply(xifti$data, is.null, FALSE)
-  if (!any(bs_present)) { 
+  if (!any(bs_present)) {
     return(0)
   } else {
     return(ncol(xifti$data[[which(bs_present)[1]]]))
@@ -306,28 +312,28 @@ ncol_xifti <- function(xifti) {
 }
 
 #' Dimensions of a \code{"xifti"}
-#' 
-#' Returns the number of rows (vertices + voxels) and columns (measurements) in 
-#'  the \code{"xifti"} data. 
-#' 
+#'
+#' Returns the number of rows (vertices + voxels) and columns (measurements) in
+#'  the \code{"xifti"} data.
+#'
 #' @inheritParams x_Param_xifti
 #' @return The number of rows and columns in the \code{"xifti"} data.
-#' 
-#' @export 
+#'
+#' @export
 #' @method dim xifti
 dim.xifti <- function(x) {
   c(nrow_xifti(x), ncol_xifti(x))
 }
 
 #' Convert a \code{"xifti"} to a matrix
-#' 
+#'
 #' Converts a \code{"xifti"} to a matrix by concatenating the data from each
 #'  brainstructure along the rows. Surfaces and metadata are discarded.
-#' 
+#'
 #' @inheritParams x_Param_xifti
 #' @param ... Unused
 #' @return The input as a matrix. Each brainstructure's data is concatenated.
-#' 
+#'
 #' @export
 #' @method as.matrix xifti
 as.matrix.xifti <- function(x, ...) {
@@ -335,41 +341,78 @@ as.matrix.xifti <- function(x, ...) {
 }
 
 #' Infer resolution from \code{"xifti"} and surfaces
-#' 
+#'
+#' Infer the numbers of vertices on each cortex of a \code{"xifti"} object.
+#'  Also supports the result of \code{\link{info_cifti}}.
+#'
 #' @inheritParams xifti_Param
 #' @param surfL Left surface
 #' @param surfR Right surface
-#' 
+#'
 #' @export
-#' 
+#'
 #' @return The inferred resolution
-#' 
+#'
 infer_resolution <- function(xifti, surfL=NULL, surfR=NULL) {
-  res <- NULL
+  res <- c(NA, NA)
+
+  # info_cifti
+  was_info <- identical(names(xifti), c("cortex", "subcort", "cifti"))
+  if (was_info) {
+    q <- template_xifti(); q$meta <- xifti; xifti <- q
+  }
+
+  # Medial wall mask.
   if (!is.null(xifti$meta$cortex$medial_wall_mask$left)) {
-    res <- length(xifti$meta$cortex$medial_wall_mask$left)
-  } else if (!is.null(xifti$meta$cortex$medial_wall_mask$right)) {
-    res <- length(xifti$meta$cortex$medial_wall_mask$right)
-  } else {
+    res[1] <- length(xifti$meta$cortex$medial_wall_mask$left)
+  }
+  if (!is.null(xifti$meta$cortex$medial_wall_mask$right)) {
+    res[2] <- length(xifti$meta$cortex$medial_wall_mask$right)
+  }
+
+  # Data/surfaces, pass 1
+  if (identical(res, c(NA, NA))) {
     if (!is.null(xifti$data$cortex_left) && !is.null(xifti$data$cortex_right)) {
       if (nrow(xifti$data$cortex_left) == nrow(xifti$data$cortex_right)) {
-        res <- nrow(xifti$data$cortex_left)
+        res[c(1,2)] <- nrow(xifti$data$cortex_left)
+      }
+    } else {
+      if (!is.null(xifti$data$cortex_left) && !is.null(surfL)) {
+        prop_mwall <- nrow(xifti$data$cortex_left) / nrow(surfL$vertices)
+        if (prop_mwall<=1 && prop_mwall >.85) { res[1] <- nrow(surfL$vertices) } else { res[1] <- nrow(xifti$data$cortex_left) }
+      }
+      if (!is.null(xifti$data$cortex_right) && !is.null(surfR)) {
+        prop_mwall <- nrow(xifti$data$cortex_right) / nrow(surfR$vertices)
+        if (prop_mwall<=1 && prop_mwall >.85) { res[2] <- nrow(surfR$vertices) } else { res[2] <- nrow(xifti$data$cortex_right) }
       }
-    } else if (!is.null(xifti$data$cortex_left) && !is.null(surfL)) {
-      prop_mwall <- nrow(xifti$data$cortex_left) / nrow(surfL$vertices)
-      if (prop_mwall<=1 && prop_mwall >.85) { res <- nrow(surfL$vertices) } else { res <- nrow(xifti$data$cortex_left) }
-    } else if (!is.null(xifti$data$cortex_right) && !is.null(surfR)) {
-      prop_mwall <- nrow(xifti$data$cortex_right) / nrow(surfR$vertices)
-      if (prop_mwall<=1 && prop_mwall >.85) { res <- nrow(surfR$vertices) } else { res <- nrow(xifti$data$cortex_right) }
     }
-    if (is.null(res)) {
-      nvL <- ifelse(is.null(surfL), Inf, nrow(surfL$vertices))
-      nvR <- ifelse(is.null(surfR), Inf, nrow(surfR$vertices))
-      res <- min(nvL, nvR)
-      if (is.infinite(res)) { res <- NULL }
+  }
+
+  # Data, pass 2
+  if (identical(res, c(NA, NA))) {
+    if (!is.null(xifti$data$cortex_left)) { res[1] <- nrow(xifti$data$cortex_left) }
+    if (!is.null(xifti$data$cortex_right)) { res[2] <- nrow(xifti$data$cortex_right) }
+  }
+
+  # Surface, pass 2
+  if (identical(res, c(NA, NA))) {
+    if (!is.null(surfL)) { res[1] <- nrow(surfL$vertices) }
+    if (!is.null(surfR)) { res[2] <- nrow(surfR$vertices) }
+  }
+
+  # Empty
+  if (!was_info) {
+    if (is.na(res[1])) {
+      if (is.null(xifti$data$cortex_left) && is.null(xifti$meta$cortex$medial_wall_mask$left)) {
+        res[1] <- 0
+      }
+    }
+    if (is.na(res[2])) {
+      if (is.null(xifti$data$cortex_right) && is.null(xifti$meta$cortex$medial_wall_mask$right)) {
+        res[2] <- 0
+      }
     }
-    # [TO DO]: if no medial wall mask exists, but a cortex has data?
   }
 
   res
-}
\ No newline at end of file
+}
